Unit ubits;
(* ******************************************************************** *)
(* K L A S S E :                                                        *)
(* -------------------------------------------------------------------- *)
(* Version     :                                                        *)
(* Autor       : http://home.tu-clausthal.de/~inas/computer/FAQ/pas/69  *)
(*               bearbeit v. S. Spolwig, OSZ Handel I, Berlin           *)
(* Beschreibung: Die Klasse stellt eine Reihe von Funktionen zur Bitma- *)
(*               nipulation in Integer-und Word - Variablen zur         *)
(*               Verfuegung                                             *)
(*                                                                      *)
(* Compiler    : Delphi 4.0                                             *)
(* Aenderungen : AUG - 00                                               *)
(* ******************************************************************** *)


Interface


type
  TMyBits = class
    Function ITstBit ( Zahl : Word; Bitnr : Integer ) : Boolean;
    Function BTstBit ( Zahl : Byte; Bitnr : Integer ) : Boolean;
    Function ISetBit ( Zahl: Word; Bitnr: Integer ) : Word;
    Function BSetBit ( Zahl: Byte; Bitnr: Integer ) : Byte;
    Function IRotateLeft ( Zahl : Word; Bits : Integer ) : Word;
    Function BRotateLeft ( Zahl : Byte; Bits : Integer ) : Byte;
    Function IRotateRight ( Zahl : Word; Bits : Integer ) : Word;
    Function BRotateRight ( Zahl : Byte; Bits : Integer ) : Byte;
    Function IDeleteBit ( Zahl : Word; Bitnr : Integer ) : Word;
    Function BDeleteBit ( Zahl : Byte; Bitnr : Integer ) : Byte;
    Function IntToBinStr ( Zahl : Word ) : String;
    Function ByteToBinStr ( Zahl : Byte ) : String;
    Function BinStrToInt ( Str1 : String ) : Word;
    Function BinStrToByte ( Str1 : String ) : Byte;
  end;

(* -------------------- B e s c h r e i b u n g -------------------------

Oberklasse    : -
Bezugsklassen : -


Methoden
--------
Die Funktionen, die mit einem "I" beginnen, werden zur Bearbeitung von Zahlen
der Typen Integer und Word benutzt, die mit einem "B" beginnen, sind fuer
die Typen Byte und Shortint gedacht.



ITstBit, BTstBit
  Auftrag: Testet die angegebene Bitposition. TRUE wird uebergeben,
            wenn das getestete Bit gesetzt ist

ISetBit, BSetbit
  Auftrag: Setzt das angegebene Bit

IRotateLeft, BRotateLeft.
  Auftrag: Rotiert die Zahl nach links. Das hoechstwertige Bit wird nachgeschoben

IRotateRight, BRotateRight
  Auftrag; Rotiert die Zahl nach rechts. Das niedrigstwertige bit wird nachgeschoben.

IDeleteBit, BDeleteBit
  Auftrag: Loescht das angegebene Bit.

IntToBinStr, ByteToBinStr
  Auftrag: Wandelt eine Zahl in einen Binaerstring

BinStrToInt, BinStrToByte
  Auftrag: Wandelt einen Binaerstring in eine Zahl

----------------------------------------------------------------------- *)

Implementation

(* ==================================================================== *)
Function TMyBits.ITstBit ( Zahl : Word; Bitnr : Integer ) : Boolean;
(* -------------------------------------------------------------------- *)
Begin
  Bitnr := Bitnr And $000F;
  Itstbit := (( Zahl Shr Bitnr ) And 1 ) = 1
End;

Function TMyBits.BTstBit ( Zahl : Byte; Bitnr : Integer ) : Boolean;
(* -------------------------------------------------------------------- *)
Begin
  Bitnr := Bitnr And $0007;
  Btstbit := (( Zahl Shr Bitnr ) And 1 ) = 1
End;

Function TMyBits.ISetBit ( Zahl : Word; Bitnr : Integer ) : Word;
(* -------------------------------------------------------------------- *)
Var I : Word;
Begin
  Bitnr := Bitnr And $000F;
  I := 1;
  Isetbit := Zahl Or ( I Shl Bitnr )
End;

Function TMyBits.BSetBit ( Zahl : Byte; Bitnr : Integer ) : Byte;
(* -------------------------------------------------------------------- *)
Var I : Byte;
Begin
  Bitnr := Bitnr And $0007;
  I := 1;
  Bsetbit := Zahl Or ( I Shl Bitnr )
End;

Function TMyBits.IRotateLeft ( Zahl : Word; Bits : Integer ) : Word;
(* -------------------------------------------------------------------- *)
Var
  Bit15 : Boolean;
  X     : Word;
  I     : Integer;
Begin
  Bits := Bits And $000F; X := Zahl;
  For I := 1 To Bits Do
  Begin Bit15 := Itstbit ( X, 15 );
    X := X Shl 1;
    If Bit15
    Then X := Isetbit ( X, 0 )
  End;
  IRotateLeft := X
End;

Function TMyBits.BRotateLeft ( Zahl : Byte; Bits : Integer) : Byte;
(* -------------------------------------------------------------------- *)
Var
  Bit7  : Boolean;
  X     : Byte;
  I     : Integer;
Begin
  Bits := Bits And $0007;
  X := Zahl;
  For I := 1 To Bits Do
  Begin Bit7 := Btstbit ( X, 7 );
    X := X Shl 1;
    If Bit7
    Then X := Bsetbit ( X, 0 )
  End;
  BRotateLeft := X
End;

Function TMyBits.IRotateRight (Zahl : Word; Bits : Integer ) : Word;
(* -------------------------------------------------------------------- *)
Var
  Bit0  : Boolean;
  X     : Word;
  I     : Integer;
Begin
  Bits := Bits And $000F;
  X := Zahl;
  For I := 1 To Bits Do
  Begin
    Bit0 := Itstbit ( X, 0 );
    X := X Shr 1;
    If Bit0
    Then X := Isetbit ( X, 15 )
  End;
  IRotateRight := X
End;

Function TMyBits.BRotateRight( Zahl : Byte; Bits : Integer ) : Byte;
(* -------------------------------------------------------------------- *)
Var
  Bit0  : Boolean;
  X     : Byte;
  I     : Integer;
Begin
  Bits := Bits And $0007;
  X    := Zahl;
  For I := 1 To Bits Do
  Begin
    Bit0 := Btstbit ( X, 0 );
    X := X Shr 1;
    If Bit0
    Then X := Bsetbit ( X, 7 );
  End;
  BRotateRight := X
End;

Function TMyBits.IDeleteBit ( Zahl : Word; Bitnr : Integer ): Word;
(* -------------------------------------------------------------------- *)
Var X : Word;
Begin
  Bitnr := Bitnr And $000F;
  X := $Fffe;
  IDeleteBit := Zahl And IRotateLeft ( X, Bitnr )
End;

Function TMyBits.BDeleteBit ( Zahl : Byte; Bitnr : Integer) : Byte;
(* -------------------------------------------------------------------- *)
Var X : Byte;
Begin
  Bitnr := Bitnr And $0007;
  X := $Fe;
  BDeleteBit := Zahl And BRotateLeft ( X, Bitnr )
End;

Function TMyBits.IntToBinStr ( Zahl : Word ) : String;
(* -------------------------------------------------------------------- *)
Var
  I : Integer;
  W : String[16];
Begin W := '0000000000000000';
For I := 15 Downto 0 Do
  If Itstbit ( Zahl, I )
  Then Insert ('1', W, 16 - I );
  IntToBinStr := W
End;


Function TMyBits.ByteToBinStr ( Zahl : Byte ) : String;
(* -------------------------------------------------------------------- *)
Var I : Integer; W : String[8];
Begin
  W := '00000000';
  For I := 7 Downto 0 Do
  If Btstbit ( Zahl,I )
  Then Insert ('1',W,8 - I );
  ByteToBinStr := W
End;

Function TMyBits.BinStrToInt ( Str1 : String ) : Word;
(* -------------------------------------------------------------------- *)
Var
  I : Integer;
  J : Integer;
  K : Word;
Begin
  K := 0;
  J := Length ( Str1 );
  If ( J > 16 )
  Then J := 16;
  For I := J Downto 1 Do
    If ( Str1[I] = '1' )
    Then K := Isetbit( K, J - I );
    BinStrToInt := K
End;

Function TMyBits.BinStrToByte ( Str1 : String ) : Byte;
(* -------------------------------------------------------------------- *)
Var
  I,J : Integer;
  K   : Byte;
Begin
  K := 0;
  J := Length ( Str1 );
  If ( J > 8 )
  Then J := 8;
  For I := J Downto 1 Do
  If ( Str1[I] = '1')
  Then K := Bsetbit ( K, J - I );
  BinStrToByte := K
End;

End.

