unit uGrafik;
(* ******************************************************************** *)
(*                                                                      *)
(* K L A S S E : TFigur, TZeichenblatt u. a.                            *)
(* -------------------------------------------------------------------- *)
(* Version     : 2.1                                                    *)
(* Autor       : (c) S. Spolwig, OSZ-Handel I, 10997 Berlin             *)
(*                                                                      *)
(* Beschreibung: Die Unit ist eine Bibliothek von Grafikklassen.        *)
(*               Sie stellt ein TZeichenblatt zur Verfgung, das in     *)
(*							 einer TImage-Komponente liegt und das den Hintergrund  *)
(*							 fr die anderen Objekte bildet.												*)
(*               Dem Objekt muss mit Zeichenblatt.SetLink(..) die Image-*)
(*							 Komponente und das aktuelle Exemplar selbst bergeben  *)
(*							 werden. 															         					*)
(*               Die anderen Klassen/Objekte beziehen sich auf ZEICHEN- *)
(*               BLATT.                                                 *)
(*               X1 und Y1 sind der Bezugspunkt jeweils links (oben),   *)
(*               beim Kreis der Mittelpunkt.                            *)
(*               Der Ursprung ist [0,0]                                 *)
(*               Die Farben sind vom Typ TColor in DELPHI.              *)
(* Compiler    : DELPHI 5.0                                             *)
(* Aenderung   : V. 0.9   13-NOV-98                                     *)
(*                  1.0   14-NOV-98   polymorphe Methoden               *)
(*                  1.01  04-DEC-98   Bug in Textpos; Farben gesetzt in *)
(*                                    Zeigen, sonst falsche werte       *)
(*                  1.02  14-JAN-99   GetBreite, GetHoehe fr Zeichblatt*)
(*                  1.03  05-JUL-99   Bug in Text.Zeigen. Richtiger Hin-*)
(*                                    grund                             *)
(*                  2.0   17-JUL-99   SetPos statt constructor Init(),  *)
(*                                    TZeichenblatt ist Unterklasse von *)
(*                                    TRechteck								          *)
(*									2.1		15-JUN-01		TZeichenblatt.SetLink (...)				*)
(*																		Damit entfllt der Zwangsexport   *)
(*																		von ZEICHENBLATT und es kann wie  *)
(*																		alle anderen behandelt werden.   	*)
(*																		Die Figuren knnen auf mehreren   *)
(*																		Images dargestellt werden.        *)
(*																		Das zuletzt aufgerufene ist immer *)
(*																		(leider) systemweit das aktuelle. *)
(*																		Zeichenblatt.Init() entfllt.			*)
(* ******************************************************************** *)

interface
uses graphics, extctrls;

type
  TFigur  = class (TObject)
    protected
    x1, y1, x2, y2 : longint;
    Farbe,
    Fuellfarbe     : Tcolor;

    public
    procedure SetPos(ax1, ay1, ax2, ay2 : longint); overload;virtual;
    function  GetXpos : longint;          virtual;
    function  GetYpos : longint;          virtual;
    procedure SetFarbe (F: TColor);       virtual;
    function  GetFarbe : TColor;          virtual;
    procedure SetFuellFarbe (F : Tcolor); virtual;
    function  GetFuellfarbe : TColor;     virtual;
    procedure PosVersetzenUm (dx, dy : longint);    virtual;
    procedure Zeigen;                     virtual;
    procedure Loeschen;                   virtual;
    procedure Entfernen;                  virtual;
  end;


  TLinie = class (TFigur)
    procedure Zeigen;     override;
  end;


  TRechteck = class (TFigur)
    procedure Zeigen;     override;
  end;


  TEllipse = class (TFigur)
    procedure Zeigen;     override;
  end;


  TKreis = class (TFigur)
    private
    Radius : longint;
    public
    procedure SetPos (ax, ay : longint); reintroduce;overload;
    procedure SetRadius (R : longint);   virtual;
    function  GetRadius : longint;       virtual;
    procedure Zeigen;                    override;
  end;


  Ttext = class (TFigur)
    private
    Text       : string;
    Schriftart : string;
    Hoehe      : longint;
    public
    procedure SetPos(ax, ay : longint); reintroduce;overload;
    procedure SetText (T : string);   	virtual;
    procedure SetFarbe (F : TColor);  	override;
    procedure SetFont (Art : string; Grad : longint); virtual;
    procedure Zeigen;                 	override;
  end;


  TZeichenblatt = class (TRechteck)
    procedure SetLink (Im : TImage; zb : Tzeichenblatt);
    function 	GetBreite : longint; 		virtual;
    function 	GetHoehe  : longint; 		virtual;
  end;

(* -------------------- B e s c h r e i b u n g -------------------------

Oberklasse    : TObject, TFigur
Bezugsklassen : TImage, TColor

Methoden   (FUER ALLE KLASSEN !!)
--------

SetPos (...)

   Auftrag: Objekt mit den Werten in Pixeln fuer die Position versehen.
            x1,y1 sind linke obere Ecke, x2, y2 rechte untere.
            Bei ELLIPSE liegt die Figur in einem Rechteck mit diesen Werten.
            Bei KREIS beziehen sich x,y auf den Mittelpunkt.
            Bei TEXT beziehen sich x,y auf das 1. Zeichen.
   vorher : -
   nachher: Objekte haben Positionwerte


Zeigen

   Auftrag: Objekt auf dem Bildschirm darstellen
   vorher : ist vorhanden
   nachher: -

Set...
  Auftrag: Attributwert fr jeweiliges Objekt schreiben
  vorher : ist vorhanden
  nachher: Attributwert ist gesetzt

Get...
  Auftrag: Attribut aus dem Objekt lesen
  vorher : Objekt ist vorhanden.
  nachher: -

PosVersetzenUm (dx, dy : longint)
  Auftrag: die Position um dx horizontal, um dy vertikal verschieben
  vorher : Objekt ist vorhanden
  nachher: Die neue Position ist alte Pos. um dx,dy erhht oder vermindert


Loeschen
   Auftrag: jeweiliges Objekt vom Bildschirm loeschen
   vorher : -
   nachher: Objekt ist geloescht

Entfernen
   Auftrag: jeweiliges Objekt aus dem Speicher loeschen
   vorher : ist vorhanden.
   nachher: Objekt aus dem Arbeitsspeicher entfernt.

 ****  zustzlich in TZEICHENBLATT   ****

SetLink (Im : TImage; Zb : TZeichenblatt);
    Auftrag: Zeichenblatt mit der Image-Komponenten und den anderen Figuren
    				 verbinden und Startwerte eintragen
    vorher : Im und Zb sind  vorhanden
    nachher: Position und Groesse sind von Image uebernommen (also gleich gross).
    				 Farbe und und FuellFarbe sind schwarz (keine Farbe)

GetBreite
   Anfrage: nach der Breite (in x-Richtung)
   vorher : ist mit einer Imagekomponente initialisiert
   nachher: Resultat ist die Breite ( = Breite der Imagekomponente )

GetHoehe
   Anfrage: nach der Hoehe (in y-Richtung)
   vorher : ist mit einer Imagekomponente initialisiert
   nachher: Resultat ist die Hoehe ( = Hoehe der Imagekomponente )

   ------------------------------------------------------------------ *)

implementation

var
	myBild 				 : TImage;
  myzeichenblatt : TZeichenblatt;

(* ============================ FIGUR  =========================== *)

procedure TFigur.SetPos(ax1,ay1,ax2,ay2 : longint);
(* --------------------------------------------------------------- *)
begin
  x1 := ax1;
  y1 := ay1;
  x2 := ax2;
  y2 := ay2;
end;

function TFigur.GetXpos : longint;
(* --------------------------------------------------------------- *)
begin
  Result := x1;
end;

function TFigur.GetYpos : longint;
(* --------------------------------------------------------------- *)
begin
  Result := Y1;
end;

procedure TFigur.SetFarbe (F : TColor);
(* --------------------------------------------------------------- *)
begin
  Farbe := F;
end;


procedure TFigur.SetFuellFarbe (F : Tcolor);
(* --------------------------------------------------------------- *)
begin
  Fuellfarbe := F;
end;


function TFigur.GetFarbe : TColor;
(* --------------------------------------------------------------- *)
begin
  Result := Farbe;
end;


function  TFigur.GetFuellfarbe : TColor;
(* --------------------------------------------------------------- *)
begin
   result := Fuellfarbe;
end;


procedure TFigur.Zeigen;
(* --------------------------------------------------------------- *)
begin
  with mybild.canvas do
  begin
    pen.color   := Farbe;
    brush.color := Fuellfarbe;
  end;
end;


procedure TFigur.PosVersetzenUm (dx, dy : longint);
(* --------------------------------------------------------------- *)
begin
  x1 := x1 + dx;
  y1 := y1 + dy;
  x2 := x2 + dx;
  y2 := y2 + dy;
end;


procedure TFigur.Loeschen;
(* --------------------------------------------------------------- *)
var af, aff : TColor;
begin
  af := GetFarbe;        // alte Farbe merken
  aff:= GetFuellFarbe;
  SetFuellFarbe (myZeichenblatt.GetFuellFarbe);
  SetFarbe 			(myZeichenblatt.GetFuellFarbe);
  Zeigen;
  SetFarbe(af);
  SetFuellfarbe(aff)
end;


procedure TFigur.Entfernen;
(* --------------------------------------------------------------- *)
begin
  Free;
end;


(* ============================ LINIE     ======================== *)

procedure TLinie.Zeigen;
(* --------------------------------------------------------------- *)
begin
  inherited Zeigen;
  with mybild.canvas do
  begin
    moveTo(x1,y1);
    lineTo(x2,y2);
  end;
end;


(* ============================ RECHTECK ========================= *)

procedure TRechteck.Zeigen;
(* --------------------------------------------------------------- *)
begin
  inherited zeigen;
  with mybild.canvas do
  begin
    rectangle(x1,y1,x2,y2);
  end;
end;

(* ============================ ELLIPSE   ======================== *)

procedure TEllipse.Zeigen;
(* --------------------------------------------------------------- *)
begin
  inherited zeigen;
  with mybild.canvas do
  begin
    ellipse(x1,y1,x2,y2);
  end;
end;


(* ============================ KREIS     ======================== *)

procedure TKreis.SetPos (ax, ay : longint);
(* --------------------------------------------------------------- *)
begin
  x1 := ax;
  y1 := ay;
end;

procedure TKreis.SetRadius (R : longint);
(* --------------------------------------------------------------- *)
begin
  Radius := R;
end;

function TKreis.GetRadius : longint;
(* --------------------------------------------------------------- *)
begin
  Result := Radius;
end;


procedure TKreis.Zeigen;
(* --------------------------------------------------------------- *)
begin
  inherited zeigen;
  with mybild.canvas do
  begin
    ellipse(Round(x1-Radius), Round(y1-Radius), Round(x1+Radius), Round(y1+Radius));
  end;
end;

(* ============================ TEXT ============================= *)

procedure Ttext.SetPos (ax, ay : longint);
(* --------------------------------------------------------------- *)
begin
  x1 := ax;
  y1 := ay;
end;

procedure  Ttext.SetText (T : string);
(* --------------------------------------------------------------- *)
begin
  text := T;
end;

procedure Ttext.SetFarbe (F : TColor);
(* --------------------------------------------------------------- *)
begin
  Farbe := F;
end;

procedure Ttext.SetFont (art : string; Grad : longint);
(* --------------------------------------------------------------- *)
begin
  Schriftart := art;
  Hoehe      := Grad;
end;

procedure  Ttext.Zeigen;
(* --------------------------------------------------------------- *)
begin
  inherited zeigen;
  with mybild.canvas do
  begin
    font.color := Farbe;
    font.name := Schriftart;
    font.Size := Hoehe;
    textout(x1,y1,text);
  end;
end;


(* ============================ ZEICHENBLATT ===================== *)

procedure TZeichenblatt.SetLink (Im : TImage; zb : TZeichenblatt);
(* --------------------------------------------------------------- *)
begin
  mybild := Im;       // an globale Var bergeben fr internen Gebrauch
  myzeichenblatt := zb;// akt. Exemplar kopieren fr internen Gebrauch

  x1 := 0;
  y1 := 0;
  x2 := myBild.width;  // auf volle Grsse anpassen
  y2 := mybild.height;
end;


function TZeichenblatt.GetBreite : longint;
(* --------------------------------------------------------------- *)
begin
  Result := x2;
end;


function TZeichenblatt.GetHoehe : longint;
(* --------------------------------------------------------------- *)
begin
  Result := y2;
end;


end.
