unit uGrafik;
(* ******************************************************************** *)
(*                                                                      *)
(* K L A S S E : TFigur, TZeichenblatt u. a.                            *)
(* -------------------------------------------------------------------- *)
(* Version     : 2.21                                                   *)
(* Autor       : (c) 1998-2004, S.Spolwig, OSZ-Handel I, 10997 Berlin   *)
(*                                                                      *)
(* Beschreibung: Die Unit ist eine Bibliothek von Grafikklassen.        *)
(*               Sie stellt ein TZeichenblatt zur Verfgung, das in     *)
(*		           einer TImage-Komponente liegt und das den Hintergrund  *)
(*		           fr die anderen Objekte bildet.                        *)
(*               Dem Objekt muss mit Zeichenblatt.SetLink(..) die Image-*)
(*		           Komponente bergeben werden.                           *)
(*               Die anderen Klassen/Objekte beziehen sich auf ZEICHEN- *)
(*               BLATT.                                                 *)
(*               X1 und Y1 sind der Bezugspunkt jeweils links (oben),   *)
(*               beim Kreis der Mittelpunkt. Der Ursprung ist [0,0].    *)
(*               Die Farben sind vom Typ TColor in DELPHI.              *)
(* Compiler    : DELPHI 5.0 / 6.0                                       *)
(*                                                                      *)
(* Aenderung   : V. 0.9   13-NOV-98                                     *)
(*                  1.0   14-NOV-98   polymorphe Methoden               *)
(*                  1.01  04-DEC-98   Bug in Textpos; Farben gesetzt in *)
(*                                    Zeigen, sonst falsche werte       *)
(*                  1.02  14-JAN-99   GetBreite, GetHoehe fr Zeichblatt*)
(*                  1.03  05-JUL-99   Bug in Text.Zeigen. Richtiger Hin-*)
(*                                    grund                             *)
(*                  2.0   17-JUL-99   SetPos statt constructor Init(),  *)
(*                                    TZeichenblatt ist Unterklasse von *)
(*                                    TRechteck	                        *)
(*		              2.1	  15-JUN-01   TZeichenblatt.SetLink (...)       *)
(*                                    Damit entfllt der Zwangsexport   *)
(*				                            von ZEICHENBLATT und es kann wie  *)
(*				                            alle anderen behandelt werden.   	*)
(*				                            Die Figuren knnen auf mehreren   *)
(*				                            Images dargestellt werden.        *)
(*				                            Das zuletzt aufgerufene ist immer *)
(*				                            (leider) systemweit das aktuelle. *)
(*				                            Zeichenblatt.Init() entfllt.     *)
(*                  2.11  15-NOV-01   Delphi6-Bug Flackern beseitigt    *)
(*                  2.12  10-OKT-02   TZeichenblatt.SetLink (...) ohne  *)
(*                                    2. Parameter                      *)
(*                  2.13  03-MAR-03   TPunkt - Vorschlag J. Frank       *)
(*                                    Rechteck.GetBreite, ..GetHoehe    *)
(*                  2.2   03-MAR-03   IstMarkiert, Ziehen,              *)
(*                                    PosVersetzenNach(..)              *)
(*                  2.21  15-MAR-04   TBild                             *)
(* ******************************************************************** *)

INTERFACE
  uses graphics, extctrls;

(* ============================  TFIGUR  ============================== *)
type
  TFigur  = class (TObject)
   protected
    x1, y1, x2, y2 : longint;
    Farbe,
    Fuellfarbe     : Tcolor;

   public
    procedure SetPos(ax1, ay1, ax2, ay2 : longint);   virtual;
    function  GetXpos : longint;                      virtual;
    function  GetYpos : longint;                      virtual;
    procedure SetFarbe (F: TColor);                   virtual;
    function  GetFarbe : TColor;                      virtual;
    procedure SetFuellFarbe (F : Tcolor);             virtual;
    function  GetFuellfarbe : TColor;                 virtual;
    procedure Entfernen;                              virtual;
    function  IstMarkiert (ax,ay : longint) : boolean;virtual;
    procedure Loeschen;                               virtual;
    procedure PosVersetzenUm (dx, dy : longint);      virtual;
    procedure PosVersetzenNach (neuX, neuY : longint);virtual;
    procedure Zeigen;                                 virtual;
    procedure Ziehen (ax, ay : longint);              virtual;
  end;

(* =============================  TLinie  ============================= *)

  TLinie = class (TFigur)
    function  IstMarkiert (ax, ay : longint):boolean; override;
    procedure Zeigen;                                 override;
  end;


(* ==================================================================== *)

  TRechteck = class (TFigur)
    function 	GetBreite : longint; 	virtual;
    function 	GetHoehe  : longint; 	virtual;
    procedure Zeigen;               override;
  end;

(* =============================  TEllipse  =========================== *)

  TEllipse = class (TFigur)
    procedure Zeigen;               override;
  end;

(* =============================  TKreis  ============================= *)

  TKreis = class (TFigur)
    private
    Radius : longint;
    public
    procedure SetPos (ax, ay : longint);              reintroduce;overload;
    procedure SetRadius (R : longint);                virtual;
    function  GetRadius : longint;                    virtual;
    function  IstMarkiert (ax,ay : longint) : boolean;override;
    procedure PosVersetzenNach (neuX, neuY : longint);override;
    procedure Zeigen;                                 override;
  end;

(* =============================  TPunkt  ============================= *)

  TPunkt = class (TFigur)
    procedure SetPos (ax, ay : longint);  reintroduce;overload;
    procedure Zeigen;                     override;
  end;

(* =============================  TText  ============================== *)

  Ttext = class (TFigur)
   private
    Text       : string;
    Schriftart : string;
    Hoehe      : longint;
   public
    procedure SetPos(ax, ay : longint);               reintroduce;overload;
    procedure SetText (T : string);   	              virtual;
    procedure SetFarbe (F : TColor);  	              override;
    procedure SetFont (Art : string; Grad : longint); virtual;
    function  IstMarkiert (ax, ay : longint):boolean; override;
    procedure PosVersetzenNach (neuX, neuY : longint);override;
    procedure Zeigen;                 	              override;
  end;

(* =============================  TBild  ============================== *)

  TBild = class (TFigur)
   protected
    Bitmap :TBitmap;
   public
    constructor Create;
    procedure SetPos(ax1, ay1, ax2, ay2 : longint); override;
    procedure Zeigen;                               override;
    procedure Loeschen;                             override;
    procedure Entfernen;                            override;
    procedure Laden(dateiname : string);            virtual;
    function  IstMarkiert (ax, ay : longint) : boolean; override;
    procedure Ziehen (ax, ay : longint);            override;
  end;

(* =============================  TZeichenblatt  ====================== *)

  TZeichenblatt = class (TRechteck)
    procedure SetLink (Im : TImage); virtual;
  end;

(* -------------------- B e s c h r e i b u n g -------------------------

Oberklasse    : TObject, TFigur
Bezugsklassen : TImage, TColor

Methoden   (FUER ALLE KLASSEN !!)
--------

SetPos (...)
  Auftrag: Objekt mit den Werten in Pixeln fuer die Position versehen.
           x1,y1 sind linke obere Ecke, x2, y2 rechte untere.
           Bei ELLIPSE liegt die Figur in einem Rechteck mit diesen Werten.
           Bei KREIS beziehen sich x,y auf den Mittelpunkt.
           Bei TEXT beziehen sich x,y auf das 1. Zeichen.
           Bei BILD wird die Ausgabeflche (Bitmap) auf dem Zeichenblatt begrenzt 
  vorher : -
  nachher: Objekte haben Positionwerte


Set...
  Auftrag: Attributwert fr jeweiliges Objekt schreiben
  vorher : ist vorhanden
  nachher: Attributwert ist gesetzt

Get...
  Anfrage: Attribut aus dem Objekt lesen
  vorher : Objekt ist vorhanden.
  nachher: -

IstMarkiert (x,y : longint): boolean;
  Anfrage: ob sich die Koordinaten fr x und y innerhalb der Flche der Figur
           liegen; bei Linie und Kreis innerhalb eines einschlieenden Rechtecks,
           bei Text 20X20 pix auf dem 1. Zeichen.
  vorher : -
  nachher: true, wenn innerhalb.

PosVersetzenNach (neuX, neuY : longint)
  Auftrag: die Position neu festlegen.
  vorher : Objekt ist vorhanden
  nachher: Attributwerte sind gesetzt


PosVersetzenUm (dx, dy : longint)
  Auftrag: die Position um dx horizontal, um dy vertikal verschieben
  vorher : Objekt ist vorhanden
  nachher: Die neue Position ist alte Pos. um dx,dy erhht oder vermindert


Loeschen
  Auftrag: jeweiliges Objekt vom Bildschirm loeschen
  vorher : -
  nachher: Objekt ist geloescht

Entfernen
  Auftrag: jeweiliges Objekt aus dem Speicher loeschen
  vorher : ist vorhanden.
  nachher: Objekt aus dem Arbeitsspeicher entfernt.


Zeigen
  Auftrag: Objekt auf dem Bildschirm darstellen
  vorher : ist vorhanden
  nachher: -


IstMarkiert
  Anfrage: ob Figur im Fokus (LMouse gedrckt) ist.
  vorher : Figur ist vorhanden
  nachher: true, wenn LMouse in aktiver Flche liegt


Ziehen (x,y: longint);
  Auftrag: Figur auf dem Bildschirm nach x,y mit LMOuse gedrckt verschieben.
           Figur bleibt dabei sichtbar.
  vorher : ist vorhanden.
  nachher: F. hat neue Position und ist angezeigt.


*****  zustzlich in TBild  *****************************

  Tbild wurde aufgrund praktischer Erwgungen fr den Unterricht eingefhrt,
  obwohl es nicht systemkonform ist. Es verhlt sich genau so wie die anderen
  Figuren, mit dem Unterschied, dass nicht eine Figur auf das Zeichenblatt
  geschrieben, sondern ein bmp.Bild kopiert wird.

Laden(dateiname : string);
  Auftrag: bmp-Bild aus Datei laden (rel. / abs. Pfade erlaubt)
  vorher : Objekt ist erzeugt und Bild ist vorhanden
  nachher:-

 ****  zustzlich in TZEICHENBLATT  *********************

SetLink (Im : TImage);
    Auftrag: Zeichenblatt mit der Image-Komponenten und den anderen Figuren
    				 verbinden und Startwerte eintragen
    vorher : Im und Zb sind  vorhanden
    nachher: Position und Groesse sind von Image uebernommen (also gleich gross).
    				 Farbe und und FuellFarbe sind schwarz (keine Farbe)

GetBreite
   Anfrage: nach der Breite (in x-Richtung)
   vorher : ist mit einer Imagekomponente initialisiert
   nachher: Resultat ist die Breite ( = Breite der Imagekomponente )

GetHoehe
   Anfrage: nach der Hoehe (in y-Richtung)
   vorher : ist mit einer Imagekomponente initialisiert
   nachher: Resultat ist die Hoehe ( = Hoehe der Imagekomponente )
------------------------------------------------------------------ *)


(* =============================================================== *)
IMPLEMENTATION
(* =============================================================== *)

var
	myBild 				 : TImage;
  myzeichenblatt : TZeichenblatt;

(* ============================ FIGUR  =========================== *)

procedure TFigur.SetPos(ax1,ay1,ax2,ay2 : longint);
(* --------------------------------------------------------------- *)
begin
  x1 := ax1;
  y1 := ay1;
  x2 := ax2;
  y2 := ay2;
end;

function TFigur.GetXpos : longint;
(* --------------------------------------------------------------- *)
begin
  Result := x1;
end;

function TFigur.GetYpos : longint;
(* --------------------------------------------------------------- *)
begin
  Result := Y1;
end;

procedure TFigur.SetFarbe (F : TColor);
(* --------------------------------------------------------------- *)
begin
  Farbe := F;
end;

procedure TFigur.SetFuellFarbe (F : Tcolor);
(* --------------------------------------------------------------- *)
begin
  Fuellfarbe := F;
end;

function TFigur.GetFarbe : TColor;
(* --------------------------------------------------------------- *)
begin
  Result := Farbe;
end;

function  TFigur.GetFuellfarbe : TColor;
(* --------------------------------------------------------------- *)
begin
   result := Fuellfarbe;
end;

procedure TFigur.Zeigen;
(* --------------------------------------------------------------- *)
begin
  with mybild.canvas do
  begin
    pen.color   := Farbe;
    brush.color := Fuellfarbe;
  end;
end;

procedure TFigur.PosVersetzenUm (dx, dy : longint);
(* --------------------------------------------------------------- *)
begin
  x1 := x1 + dx;
  y1 := y1 + dy;
  x2 := x2 + dx;
  y2 := y2 + dy;
end;

procedure TFigur.PosVersetzenNach (neuX, neuY : longint);
(* --------------------------------------------------------------- *)
var breite,
    hoehe  : longint;

begin
  breite := x2 - x1;
  hoehe  := y2 - y1;

  x1 := neuX - breite DIV 2;
  y1 := neuY - hoehe DIV 2;
  x2 := x1 + breite;
  y2 := y1 + hoehe;
end;

procedure TFigur.Loeschen;
(* --------------------------------------------------------------- *)
var af, aff : TColor;
begin
  af := GetFarbe;        // alte Farbe merken
  aff:= GetFuellFarbe;
  SetFuellFarbe (myZeichenblatt.GetFuellFarbe);
  SetFarbe(myZeichenblatt.GetFuellFarbe);
  Zeigen;
  SetFarbe(af);
  SetFuellfarbe(aff)
end;

function  TFigur.IstMarkiert (ax, ay : longint) : boolean;
(* --------------------------------------------------------------- *)
var
  xo,yo,xu,yu : longint;

begin
  xo := x1;
  yo := y1;
  xu := x2;
  yu := y2;
  if ((ax >= xo) and (ax <= xu) and (ay >= yo) and (ay <= yu))
  then Result := true
  else Result := false;
end;

procedure TFigur.Ziehen (ax, ay : longint);
(* --------------------------------------------------------------- *)
begin
  Loeschen;
  PosVersetzenNach(ax,ay);
  Zeigen;
end;

procedure TFigur.Entfernen;
(* --------------------------------------------------------------- *)
begin
  Free;
end;

(* ============================ LINIE     ======================== *)

procedure TLinie.Zeigen;
(* --------------------------------------------------------------- *)
begin
  inherited Zeigen;
  with mybild.canvas do
  begin
    moveTo(x1,y1);
    lineTo(x2,y2);
  end;
end;

function TLinie.IstMarkiert (ax, ay : longint):boolean;
(* --------------------------------------------------------------- *)
var R : TRechteck;
begin
  R := TRechteck.Create;
  R.SetPos(x1-5, y1-5, x2+5, y2+5);
  if R.IstMarkiert(ax,ay)
  then Result := true
  else Result := false;
  R.Free;
end;

(* ============================ RECHTECK ========================= *)

function TRechteck.GetBreite : longint;
(* --------------------------------------------------------------- *)
begin
  Result := x2 - x1;
end;

function TRechteck.GetHoehe : longint;
(* --------------------------------------------------------------- *)
begin
  Result := y2 - y1;
end;

procedure TRechteck.Zeigen;
(* --------------------------------------------------------------- *)
begin
  inherited zeigen;
  with mybild.canvas do
  begin
    rectangle(x1,y1,x2,y2);
  end;
end;

(* ============================ ELLIPSE   ======================== *)

procedure TEllipse.Zeigen;
(* --------------------------------------------------------------- *)
begin
  inherited zeigen;
  with mybild.canvas do
  begin
    ellipse(x1,y1,x2,y2);
  end;
end;

(* ============================ KREIS     ======================== *)

procedure TKreis.SetPos (ax, ay : longint);
(* --------------------------------------------------------------- *)
begin
  x1 := ax;
  y1 := ay;
end;

procedure TKreis.SetRadius (R : longint);
(* --------------------------------------------------------------- *)
begin
  Radius := R;
end;

function TKreis.GetRadius : longint;
(* --------------------------------------------------------------- *)
begin
  Result := Radius;
end;

procedure TKreis.Zeigen;
(* --------------------------------------------------------------- *)
begin
  inherited zeigen;
  with mybild.canvas do
  begin
    ellipse(Round(x1-Radius), Round(y1-Radius), Round(x1+Radius), Round(y1+Radius));
  end;
end;

function  TKreis.IstMarkiert (ax, ay : longint) : boolean;
(* --------------------------------------------------------------- *)
var
  xo,yo,xu,yu : longint;

begin
  xo := x1 - Radius;
  yo := y1 - Radius;
  xu := x1 + Radius;
  yu := y1 + Radius;
  if ((ax >= xo) and (ax <= xu) and (ay >= yo) and (ay <= yu))
  then Result := true
  else Result := false;
end;

procedure TKreis.PosVersetzenNach (neuX, neuY : longint);
(* --------------------------------------------------------------- *)
begin
  x1 := neuX;
  y1 := neuY;
end;

(* ============================ TEXT ============================= *)

procedure Ttext.SetPos (ax, ay : longint);
(* --------------------------------------------------------------- *)
begin
  x1 := ax;
  y1 := ay;
end;

procedure  Ttext.SetText (T : string);
(* --------------------------------------------------------------- *)
begin
  text := T;
end;

procedure Ttext.SetFarbe (F : TColor);
(* --------------------------------------------------------------- *)
begin
  Farbe := F;
end;

procedure Ttext.SetFont (art : string; Grad : longint);
(* --------------------------------------------------------------- *)
begin
  Schriftart := art;
  Hoehe      := Grad;
end;

procedure  Ttext.Zeigen;
(* --------------------------------------------------------------- *)
begin
  inherited zeigen;
  with mybild.canvas do
  begin
    font.color := Farbe;
    font.name := Schriftart;
    font.Size := Hoehe;
    textout(x1,y1,text);
  end;
end;


procedure TText.PosVersetzenNach (neuX, neuY : longint);
(* --------------------------------------------------------------- *)
begin
  x1 := neuX;
  y1 := neuY;
end;


function TText.IstMarkiert (ax, ay : longint):boolean;
(* --------------------------------------------------------------- *)
var R : TRechteck;
begin
  R := TRechteck.Create;
  R.SetPos(x1-5, y1-5, x1+25, y1+15);
  R.SetFarbe(myZeichenblatt.GetFuellFarbe);
  R.SetFuellFarbe(myZeichenblatt.GetFuellFarbe);
  if R.IstMarkiert(ax,ay)
  then Result := true
  else Result := false;
  R.Free;
end;

(* ============================ PUNKT     ======================== *)

procedure TPunkt.SetPos (ax, ay : longint);
(* --------------------------------------------------------------- *)
begin
  x1 := ax;
  y1 := ay;
end;

procedure TPunkt.Zeigen;
(* --------------------------------------------------------------- *)
begin
  inherited Zeigen;
  with mybild.canvas do
  begin
    pixels[x1,y1] := Farbe;
  end;
end;

(* ============================ TBild ============================ *)

constructor TBild.Create;
(* --------------------------------------------------------------- *)
begin
  inherited;
  Bitmap := TBitmap.Create;
end;


procedure TBild.SetPos(ax1, ay1, ax2, ay2 : longint);
(* --------------------------------------------------------------- *)
begin
  x1 := ax1;
  y1 := ay1;
  x2 := ax2;
  y2 := ay2;

  Bitmap.Width  := ax2;  // Bildgroesse auf Parameter setzen
  Bitmap.Height := ay2;
end;


procedure TBild.Zeigen;
(* --------------------------------------------------------------- *)
begin
  mybild.Canvas.Draw(x1, y1,Bitmap);
end;


procedure TBild.Loeschen;
(* --------------------------------------------------------------- *)
var
  dummy   : TRechteck;
  dx2,dy2 : integer;
begin
  dx2 := self.Bitmap.Width;
  dy2 := self.Bitmap.Height;

  dummy := TRechteck.Create;
  dummy.SetPos(x1,y1,x1+dx2,y1+dy2);
  dummy.SetFarbe(myZeichenblatt.GetFuellfarbe);
  dummy.SetFuellFarbe(myZeichenblatt.GetFuellfarbe);

  mybild.Canvas.CopyMode := cmWhiteness;
  mybild.Canvas.Draw(x1, y1,Bitmap);
  mybild.Canvas.CopyMode :=cmSrcCopy;

  dummy.Zeigen;
  dummy.Free;
end;


procedure TBild.Entfernen;
(* --------------------------------------------------------------- *)
begin
  Bitmap.Free;
  inherited;
end;

procedure TBild.Laden(dateiname : string);
(* --------------------------------------------------------------- *)
begin
  BitMap.LoadFromFile(dateiname);
end;


function TBild.IstMarkiert (ax, ay : longint):boolean;
(* --------------------------------------------------------------- *)
var
  R : TRechteck;
  dx2,
  dy2 : integer;
begin
  dx2 := self.Bitmap.Width;
  dy2 := self.Bitmap.Height;

  R := TRechteck.Create;
  R.SetPos(x1-5, y1-5, x1+dx2, y1+dy2);
  if R.IstMarkiert(ax,ay)
  then Result := true
  else Result := false;

  R.Free;
end;


procedure TBild.Ziehen (ax, ay : longint);
(* --------------------------------------------------------------- *)
begin
  if IstMarkiert(ax,ay) then
  begin
  Loeschen;
  PosVersetzenNach(ax,ay);
  Zeigen;
  end;
end;

(* ============================ ZEICHENBLATT ===================== *)

procedure TZeichenblatt.SetLink (Im : TImage);
(* --------------------------------------------------------------- *)
begin
  im.Stretch := true;  // verhindert das Flackern bei Delphi 6
  mybild := Im;        // an globale Var bergeben fr internen Gebrauch
  myzeichenblatt := self;// akt. Exemplar kopieren fr internen Gebrauch

  x1 := 0;
  y1 := 0;
  x2 := myBild.width;  // auf volle Grsse anpassen
  y2 := mybild.height;
end;

end.

