UNIT uStrListe;
(* ******************************************************************** *)
(*                                                                      *)
(* K L A S S E   : TStrListe                                            *)
(* -------------------------------------------------------------------- *)
(* Version       : 1.3                                                  *)
(* Autor         : S. Spolwig, OSZ-Handel I, 10997 Berlin               *)
(*                                                                      *)
(* Aufgabe       : Allgemeine statische Liste zur Verwaltung von        *)
(*                 strings.                                             *)
(*                 Zum Bewegen in der Liste ist intern eine Listenmarke *)
(*                 mitgefuehrt, die die jeweils aktuelle Listenposition *)
(*                 bezeichnet. Alle Operationen werden auf der aktuellen*)
(*                 Position ausgefuehrt.                                *)
(*                 (SLISTE entspricht SLISTEN in Turbo-Pascal.)         *)
(*                                                                      *)
(* Compiler      : MSDOS - Turbo-Pascal 7.0 - Delphi 3.0                *)
(* Aenderung     : V. 1.0   - 20-AUG-97                                 *)
(*                    1.02  - 03-SEP-98   Load, Store                   *)
(*                    1.03  - 06-Okt-98   SearchElement                 *)
(*                    1.2   - 01-JAN-99   als Stringliste               *)
(*                                        GetSuchPos,                   *)
(*                    1.3   - 04-JAN-99   GetSuchPosBin; RekursivDurchs *)
(*                                        Sort                          *)
(*                            24-FEB-00   an SListe angepasst           *)
(*                    1.32    09-FEB-02   procedure Init, SetElement    *)
(* ******************************************************************** *)

INTERFACE
(* ==================================================================== *)

const
   MAXLAENGE = 121000;

type
  TElement = string;

  TStrListe  = class (TObject)
   private
    procedure quicksort(anfang, ende: integer); virtual;
   protected
    Kollektion  : array [0..MAXLAENGE + 1] of TElement;
    ListenLaenge,                 // Anzahl der belegten Elemente
    AktuellePos : integer;

   public
    constructor Create;  virtual;
    procedure Init;      virtual;
    procedure First;     virtual;
    procedure Next ;     virtual;
    procedure previous;  virtual;
    procedure Last ;     virtual;

    function  GetElement : TElement;         virtual;
    procedure SetElement (Elem : TElement);  virtual;
    procedure Append (Elem : TElement);      virtual;
    procedure Remove;                        virtual;
    procedure RemoveAll;                     virtual;
    function  SearchFor (Elem : TElement) : integer; virtual;
    procedure Sort;                          virtual;

    function  GetLen  : integer;             virtual;
    function  EoList  : boolean;             virtual;
    function  IsFirst : boolean;             virtual;
    function  IsLast  : boolean;             virtual;
    function  IsEmpty : boolean;             virtual;
    function  IsFull  : boolean;             virtual;

  end;
(*  ----------------- B e s c h r e i b u n g -------------------------

Oberklasse     : TObject
Bezugsklassen  : -

Methoden
--------

Create
  Auftrag : leere L. erzeugen
  vorher  : -
  nachher : ListenLaenge ist 0, AktuellePos ist 0.

Init
  Auftrag : L. init.
  vorher  : L. ist erzeugt
  nachher : ListenLaenge ist 0, AktuellePos ist 0.


First
  Auftrag : Listenmarke auf das 1. (vorhandene) Element setzen
  vorher  : Die Liste ist nicht leer
  nachher : Aktuelle Position ist 1.
            Ist die Liste leer , geschieht nichts


Next
  Auftrag : Die Listenmarke auf das naechste Element setzen.
  vorher  : Die Liste ist nicht leer und akt. Pos ist nicht letztes El.
  nachher : Die Marke steht auf der neuen aktuellen Position
            ist die Liste leer oder EoList erreicht, geschieht nichts.


Previous
  Auftrag : Die Listenmarke um eins auf das vorhergehende Element zuruecksetzen
  vorher  : Die Liste ist nicht leer und akt. Pos ist nicht 1. Element.
  nachher : Die Marke steht auf der neuen aktuellen Position.
            Ist die Liste leer oder die aktuelle Position war 1, geschieht nichts.

Last
  Auftrag : Listenmarke auf das letzte Element der Liste setzen
  vorher  : Die Liste ist nicht leer.
  nachher : Letztes vorhandenes Element ist aktuelle Position


GetElement
  Anfrage : Zugriff auf das aktuelle Element zum Lesen
  vorher  : Die Liste ist nicht leer.
  nachher : GetElement liefert das aktuelle Element.


SetElement (Elem : TElement)
  Auftrag : Zugriff auf das aktuelle Element zum Schreiben
  vorher  : Die Liste ist initialisiert.
  nachher : Elem ist an aktuelle Position uebergeben


Append (var Elem : TElement)
  Auftrag : Neues Element am Ende der Liste erzeugen und Elem uebergeben.
            Wenn die Liste leer war, ist das neue El. das erste.
  vorher  : Die Liste ist initialisiert
  nachher : Listenlaenge ist um eins erhoeht. Ist MAXLISTE erreicht,
            geschieht nichts.

Remove
  Auftrag: Das aktuelle Element loeschen.
  vorher : Die Liste ist nicht leer.
  nachher: Das aktuelle Element ist aus der Liste entfernt. Die Listen-
           laenge ist um eins vermindert. AktuellePos ist 0.

EoList
  Anfrage: ob das Ende der Liste erreicht ist.
  vorher : Die Liste ist initialisiert.
  nachher: True, wenn Ende der Liste erreicht ist. Die Listenmarke
           steht hinter dem letzten belegten Element.


IsEmpty
  Anfrage: ob Liste leer ist
  vorher : Die Liste ist initialisiert.
  nachher: True, wenn die Liste leer ist.


GetLen : integer;
  Anfrage: gibt Listenlaenge zurueck
  vorher : Liste ist initialisiert
  nachher: -

SearchFor (Elem : TElemt) : integer;
  Anfrage: ob Elem in der Liste enthalten ist
  vorher: -
  nachher: liefert Position in der Liste. Wenn nicht vorhanden, dann 0
  
Sort
  Auftrag: Liste sortieren
  vorher : -
  nachher: Liste ist aufsteigend sortiert

----------------------------------------------------------------------- *)


IMPLEMENTATION
(* ==================================================================== *)

constructor TStrListe.Create;
(* -------------------------------------------------------------------- *)
begin
  inherited Create;
  Init;
end;


procedure TStrListe.Init;
(* -------------------------------------------------------------------- *)
var i : integer;
begin
  for i := 0 to MAXLAENGE +1 do
  begin
    Kollektion[i]  := '';
  end;
  ListenLaenge:= 0;
  AktuellePos := 0;
end;


procedure TStrListe.First;
(* -------------------------------------------------------------------- *)
begin
  If NOT IsEmpty
  then AktuellePos := 1;
end;


procedure TStrListe.Next;
(* -------------------------------------------------------------------- *)
Begin
  if NOT (IsEmpty OR IsLast)  // EoList
  then AktuellePos := AktuellePos + 1;
end;


procedure TStrListe.Previous;
(* -------------------------------------------------------------------- *)
begin
  if NOT (IsEmpty or IsFirst)
  then AktuellePos := AktuellePos - 1;
end;


procedure TStrListe.Last;
(* -------------------------------------------------------------------- *)
begin
  if NOT (IsEmpty)
  then AktuellePos := ListenLaenge ;
end;


function TStrListe.GetElement : TElement;
(* -------------------------------------------------------------------- *)
begin
  Result := Kollektion[AktuellePos];
end;


procedure TStrListe.SetElement (Elem : TElement);
(* -------------------------------------------------------------------- *)
begin
  Kollektion[AktuellePos] := Elem;
end;


procedure TStrListe.Append (Elem : TElement);
(* -------------------------------------------------------------------- *)
begin
  if NOT IsFull
  then
    begin
      AktuellePos := Listenlaenge + 1;
      SetElement(Elem);
      inc (ListenLaenge);
    end;
end;


function  TStrListe.SearchFor (Elem : TElement) : integer;
(* -------------------------------------------------------------------- *)
begin
  Result     := 0;   // nicht gefunden!
  First;
  while (Result = 0) AND (NOT EoList) do
  begin
     if Kollektion[AktuellePos] = Elem
     then Result := AktuellePos
     else inc(AktuellePos);   //Next;
  end;
end;


procedure TStrListe.Remove;
(* -------------------------------------------------------------------- *)
var
  i    : Cardinal;

begin
  if NOT IsEmpty
  then
    begin
      for i := aktuellepos to ListenLaenge do
      begin
        Kollektion[i] := Kollektion[i+1];
      end;

      Kollektion[ListenLaenge] := '';
      dec(ListenLaenge);
      AktuellePos := 0;
    end;
end;  (* remove *)


procedure TStrListe.RemoveAll;
(* -------------------------------------------------------------------- *)
begin
  if NOT IsEmpty
  then Init;       // der Schrott bleibt im Speicher
end;

function TStrListe.GetLen : integer ;
(* -------------------------------------------------------------------- *)
begin
  Result := ListenLaenge;
end;


function TStrListe.EoList : boolean;
(* -------------------------------------------------------------------- *)
begin
  if AktuellePos = (ListenLaenge+1)
  then Result := true
  else Result := false;
end;


function TStrListe.IsFirst : boolean;
(* -------------------------------------------------------------------- *)
begin
  if AktuellePos = 1
  then Result := true
  else Result := false;
end;


function TStrListe.IsLast : boolean;
(* -------------------------------------------------------------------- *)
begin
  if AktuellePos = ListenLaenge
  then Result := true
  else Result := false;
end;


function TStrListe.IsEmpty : boolean ;
(* -------------------------------------------------------------------- *)
begin
  IsEmpty := false;
  if ListenLaenge = 0
  then IsEmpty := true;
end;

function TStrListe.IsFull : boolean ;
(* -------------------------------------------------------------------- *)
begin
  Result := false;
  if ListenLaenge = MAXLAENGE
  then Result := true;
end;


procedure TStrListe.quicksort(anfang, ende: integer);
(* -------------------------------------------------------------------  *)
var linkerzeiger,
    rechterzeiger : integer;
    mitte         : string;

   procedure swap(var a,b : string);
   (* ---------------------------  *)
   var
      hilf : string;

   begin
    hilf := a;
    a    := b;
    b    := hilf
   end;

begin
  linkerzeiger  := anfang;
  rechterzeiger := ende;
  mitte         := Kollektion[(linkerzeiger + rechterzeiger) div 2];

  repeat
   while (Kollektion[linkerzeiger] < mitte) do
   begin
    linkerzeiger := linkerzeiger + 1;
   end;

   while (Kollektion[rechterzeiger] > mitte) do
   begin
     rechterzeiger := rechterzeiger-1;
   end;

   if linkerzeiger <= rechterzeiger
   then
     begin
       swap(Kollektion[linkerzeiger], Kollektion[rechterzeiger]);
       linkerzeiger := linkerzeiger + 1;
       rechterzeiger := rechterzeiger - 1;
     end;  (* if *)
  until rechterzeiger < linkerzeiger;

  if anfang < rechterzeiger
  then quicksort(anfang, rechterzeiger);

  if linkerzeiger < ende
  then quicksort(linkerzeiger, ende);
end; (* quicksort *)


procedure TStrListe.Sort;
(* -------------------------------------------------------------------- *)
begin
  quicksort(1,ListenLaenge);
end;


END.
